<?php
App::uses('AppController', 'Controller');
App::uses('Sanitize', 'Utility');

/**
 * Class FilesController
 *
 * @property File $File
 */
class FilesController extends AppController
{
    /**
    * Name of the Controller, 'Files'
    */
	public $name = 'Files';

    /**
    * array of permissions for this page
    */
	private $permissions;

    /**
    * In this beforeFilter we will get the permissions to be used in the view files
    */
	public function beforeFilter()
	{
		parent::beforeFilter();

		$this->permissions = $this->getPermissions();

		if ($this->request->action == 'admin_add' || $this->request->action == 'admin_edit' || strstr($this->request->action, 'add'))
		{
			$this->loadModel('Theme');

			$file_types = array_combine($this->Theme->file_types_editable, $this->Theme->file_types_editable);

			$media_list = $this->File->Media->find('list');
			$zoom_levels = $this->File->getZoomLevels();

            $this->set(compact('file_types', 'media_list', 'zoom_levels'));
		}

        if ($this->request->action == 'admin_add')
            $this->Security->validatePost = false;
	}

    /**
     * Admin Index
     * Returns a paginated index of Files
     *
     * @return array
     */
	public function admin_index()
	{
		$conditions = array();

		if (isset($this->request->named['trash']))
	        $conditions['File.only_deleted'] = true;

	    if ($this->permissions['any'] == 0)
	    	$conditions['User.id'] = $this->Auth->user('id');

        $this->Paginator->settings = array(
            'conditions' => $conditions,
            'contain' => array(
            	'User'
            )
        );
        
        $this->request->data = $this->Paginator->paginate('File');
	}

	/**
	 * Admin Add
	 *
	 * @return void
	 */
	public function admin_add()
	{
        if (!empty($this->request->data))
        {
	        $this->File->create();

        	$this->request->data['File']['user_id'] = $this->Auth->user('id');

            if ($this->File->saveAll( $this->File->beforeAdd($this->request->data) ))
            {
                $this->Session->setFlash('Your file has been upload.', 'success');
                $this->redirect(array('action' => 'index'));
            } else {
                $this->Session->setFlash('Unable to upload your file.', 'error');
            }
        }
	}

	/**
	 * Admin Add Folder
	 * Method that allows user to upload a folder of files to import.
	 *
	 * @return void
	 */
	public function admin_add_folder()
	{
		if (!empty($this->request->data))
		{
			$files = $this->File->uploadFolder($this->Auth->user('id'), $this->request->data['File']);

			if ($this->File->saveAll( $files ))
			{
				$this->Session->setFlash('Your folder of files has been imported.', 'success');
				$this->redirect(array('action' => 'index'));
			} else {
				$this->Session->setFlash('Unable to import your folder of files.', 'error');
			}
		}

		// Gets list of folders that are eligible (writable, in specified location)
		$folders = $this->File->getFolders();
		$this->set(compact('folders'));
	}

    /**
     * Admin Edit
    * Before POST, sets request data to form
    *
    * After POST, flash error or flash success and redirect to index
    *
    * @param integer $id id of the database entry, redirect to index if no permissions
    * @return array
    */
	public function admin_edit($id)
	{
		$this->File->id = $id;

        if (!empty($this->request->data))
        {
            $this->request->data['File']['user_id'] = $this->Auth->user('id');

            if ($this->File->saveAll($this->request->data))
            {
                $this->Session->setFlash('Your file has been updated.', 'success');
                $this->redirect(array('action' => 'index'));
            } else {
                $this->Session->setFlash('Unable to update your file.', 'error');
            }
        }

        $data = $this->File->find('first', array(
        	'conditions' => array(
        		'File.id' => $id
        	),
        	'contain' => array(
        		'Media',
        		'User'
        	)
        ));
		$this->hasAccessToItem($data);

        $file = WWW_ROOT.
        		$data['File']['dir'].
        		$data['File']['filename'];

        if (strstr($data['File']['mimetype'], 'image'))
        {
            $data['info'] = getimagesize($file);
        }
        else
        {
            $this->set('file_contents', file_get_contents($file));
        }

		$this->request->data = $data;
	}

    /**
     * Admin Delete
     * If item has no delete time, then initial deletion is to the trash area (making it in-active on site, if applicable)
     *
     * But if it has a deletion time, meaning it is in the trash, deleting it the second time is permanent.
     *
     * @param integer $id id of the database entry, redirect to index if no permissions
     * @return void
     */
    public function admin_delete($id)
    {
        $this->File->id = $id;

	    $data = $this->File->findById($id);
	    $this->hasAccessToItem($data);

	    $permanent = $this->File->remove($data);

	    $this->Session->setFlash('The file `' . $data['File']['filename'] . '` has been deleted.', 'success');

	    if ($permanent)
	    {
		    $this->redirect(array('action' => 'index', 'trash' => 1));
	    } else {
		    $this->redirect(array('action' => 'index'));
	    }
    }

    /**
     * Admin Restore
     * Restoring an item will take an item in the trash and reset the delete time
     *
     * This makes it live wherever applicable
     *
     * @param integer $id ID of database entry, redirect if no permissions
     * @return void
     */
    public function admin_restore($id)
    {
        $this->File->id = $id;

	    $data = $this->File->findById($id);
	    $this->hasAccessToItem($data);

        if ($this->File->restore())
        {
            $this->Session->setFlash('The file `' . $data['File']['filename'] . '` has been restored.', 'success');
        } else {
            $this->Session->setFlash('The file `' . $data['File']['filename'] . '` has NOT been restored.', 'error');
        }

        $this->redirect(array('action' => 'index'));
    }

	/**
	 * Admin Json List
	 *
	 * @return CakeResponse
	 */
	public function admin_json_list()
	{
		$this->view = null;
		$this->disable_parsing = true;

		if (!empty($this->request->data['q'])) {
			$q = Sanitize::clean($this->request->data['q'], array(
				'encode' => true,
				'remove_html' => true
			));
		} else {
			$q = '';
		}

		$conditions = array();
		$conditions['File.mimetype LIKE'] = '%image%';

		if (!empty($this->request->data['q'])) {
			$conditions['File.filename LIKE'] = '%' . $q . '%';
		}

		if (!empty($this->request->data['order_by'])) {
			$order_by = $this->request->data['order_by'];
		} else {
			$order_by = 'created DESC';
		}

		$this->Paginator->settings = array(
			'conditions' => $conditions,
			'order' => $order_by,
			'limit' => 9
		);

		$images = $this->File->parseMediaModal($this->Paginator->paginate('File'));

		$paginator_data = $this->request['paging']['File'];

		$start = 0;

		if ($paginator_data['count'] > 1)
			$start = (($paginator_data['page'] - 1) * $paginator_data['limit'] + 1);

		$end = $start + $paginator_data['limit'] - 1;
		if ($paginator_data['count'] < $end) {
			$end = $paginator_data['count'];
		}

		$pages = array();
		if (!empty($paginator_data['pageCount'])) {
			for($i = 1; $i <= $paginator_data['pageCount']; $i++) {
				$pages[$i] = $i;
			}
		}

		$body = array(
			'results' => $images,
			'count' => $paginator_data['count'],
			'q' => $q,
			'paginator' => $paginator_data,
			'start' => $start,
			'end' => $end,
			'pages' => $pages,
			'images_path' => WWW_ROOT
		);

		return $this->_ajaxResponse(array('body' => $body), array(), 'json');
	}
}